
${parameter:-'}'}
${parameter:-\}}

${parameter-'}'}
${parameter-\}}

${parameter:='}'}
${parameter:=\}}

${parameter='}'}
${parameter=\}}

${parameter:?'}'}
${parameter:?\}}

${parameter?'}'}
${parameter?\}}

${parameter:+'}'}
${parameter:+\}}

${parameter+'}'}
${parameter+\}}
